<?php 
	
	// line indent for printing XML trees
	$debug_xml_indent = 0;
	$ignored_classes = array();
	
	require_once("common.php");
	
	function load_ignored_classes () {
		global $ignored_classes;
		
		$root = GetDocumentRoot();
		$handle = fopen("$root/ignored_classes.txt","r");
		if ($handle) {
			$buffer = fread($handle, 12 * 1024); //filesize($handle)
			$lines = explode("\r", $buffer);

			foreach ($lines as $key => $value) {
				array_push($ignored_classes, $value);
			}
			fclose($handle);
		}
	}
	
	function print_xml ($string) {
		global $debug_xml_indent;
		$indent = "";
		
		// build indent level string
		for ($x = 0; $x <= $debug_xml_indent; $x++) {
			$indent .= "	";
		}
		
		WriteError($string);
		//print("$indent$string");
	}
	
	function same_class_name ($string1, $string2) { 
		return ((strtolower($string1)) == (strtolower($string2))); 
	} 

	function get_child_classes ($parent) { 
		$all_classes = get_declared_classes(); 
		$children = array(); 
		foreach ($all_classes as $candidate) { 
		if (same_class_name($parent, 
			get_parent_class($candidate)) && !same_class_name($parent, $candidate)) { 
				array_push($children, $candidate); 
			} 
		} 
	
		return($children); 
	} 

	function print_class_tree_aux ($parent, $level) { 
		
		/*
		// indent for class depth
		for ($x = 0; $x < $level; $x++) { 
			print("    "); 
		} 
		*/

		$children = get_child_classes($parent); 
		
		if (count($children) > 0 or $level > 0) {
			
			print_xml("<class_node>");
			print_xml("<name>$parent</name>");
		
			foreach ($children as $child) { 
				print_class_tree_aux($child, $level + 1); 
			} 
			
			print_xml("</class_node>");
		}
		
	}

	// prints a XML tree showing the class hierarchy
	function class_print_tree () { 
		global $ignored_classes;
		$all_classes = get_declared_classes(); 
		
		print_xml("<class_tree>"); 
		foreach ($all_classes as $candidate) { 
			if (!in_array($candidate, $ignored_classes)) {
				if (!get_parent_class($candidate)) { 
					print_class_tree_aux($candidate, 0); 
				} 
			}
		} 
		print_xml("</class_tree>"); 
	} 
	
	function print_ancestry ($class_name) { 
		global $debug_xml_indent;
		
		$debug_xml_indent++;

		print_xml("<class_ancestry>"); 
		print_ancestry_aux($class_name); 
		print_xml("</class_ancestry>"); 
		
		$debug_xml_indent--;
	} 

	function print_ancestry_aux ($class_name) { 
		print_xml("<class_node>");
		print_xml("		<name>$class_name</name>"); 
		print_xml("</class_node>");
		
		if ($parent = get_parent_class($class_name)) {
			print_ancestry_aux($parent); 
		} 
	} 

	// prints a debug sheet in XML format on the entire class hierarchy
	function class_inspect_all () {
		$classes = get_declared_classes();
		global $ignored_classes;
		
		print_xml("<classes> \r");
		
		foreach ($classes as $value) {
			// test against a list of classes we don't
			// care about, like built-in PHP 5 classes.
			// apply in class_print_tree() also
			
			if (!in_array($value, $ignored_classes)) {
				class_inspect($value);
			}
		}
		
		print_xml("</classes> \r");
	}

	// prints a debug sheet in XML format on the requested class
	function class_inspect ($inClass) {
		
		$methods = get_class_methods($inClass);
		
		if (count($methods) > 0) {
			
			print_xml("<class_description>");
			print_xml("<name>$inClass</name>");
				
			// print ancestory
			print_ancestry($inClass);
				
			// print class methods
			print_xml("<class_methods>");
			XMLFormatArray($methods);
			print_xml("</class_methods>");
			
			// print class instance vars
			$instanceVars = get_class_vars($inClass);
			if (count($instanceVars) > 0) {
				print_xml("<class_instance_vars>");
				XMLFormatArray($instanceVars);
				print_xml("</class_instance_vars>");
			}
			
			// close the class node
			print_xml("</class_description> \r");
		}
	}
	
	/*
	load_ignored_classes();
	if (in_array("DOMNameList", $ignored_classes)) {
		print("got it!");
	}
	*/
	//print_r($ignored_classes);
?>