<?php 

/**
 * Recursively delete the files in a directory via FTP.
 *
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.0
 * @link        http://aidanlister.com/repos/v/function.ftp_rmdirr.php
 * @param       resource $ftp_stream   The link identifier of the FTP connection
 * @param       string   $directory    The directory to delete
 */

function ftp_rmdirr($ftp_stream, $directory)
{
    // Sanity check
    if (!is_resource($ftp_stream) ||
        get_resource_type($ftp_stream) !== 'FTP Buffer') {
 
        return false;
    }
 
    // Init
    $i             = 0;
    $files         = array();
    $folders       = array();
    $statusnext    = false;
    $currentfolder = $directory;
 
    // Get raw file listing
    $list = ftp_rawlist($ftp_stream, $directory, true);
	
    // Iterate listing
    foreach ($list as $current) {
        
        // An empty element means the next element will be the new folder
        if (empty($current)) {
            $statusnext = true;
            continue;
        }
 
        // Save the current folder
        if ($statusnext === true) {
            $currentfolder = substr($current, 0, -1);
            $statusnext = false;
            continue;
        }
 
        // Split the data into chunks
        $split = preg_split('[ ]', $current, 9, PREG_SPLIT_NO_EMPTY);
        $entry = $split[8];
        $isdir = ($split[0]{0} === 'd') ? true : false;
 
        // Skip pointers
        if ($entry === '.' || $entry === '..') {
			continue;
        }
 
        // Build the file and folder list
		if ($isdir === true) {
            $folders[] = $currentfolder . '/' . $entry;
        } else {
            $files[] = $currentfolder . '/' . $entry;
        }
 
    }
 
    // Delete all the files
	foreach ($files	as $file) {
		ftp_delete($ftp_stream, $file);
		print("Deleted $file from the directory $directory.\n");
    }
 
    // Delete all the directories
    // Reverse sort the folders so the deepest directories are unset first
	rsort($folders);
	foreach ($folders as $folder) {
		ftp_rmdir($ftp_stream, $folder);
    }
 
    // Delete the final folder and return its status
	return ftp_rmdir($ftp_stream, $directory);
}
	
class TFTP {
	
	var $connection;
	var $user;
	var $password;
	var $error;
	
	function __destructor() {
		$this->Status("closing connection...");
		ftp_close($this->connection);
	}
	
	function Status ($message) {
		print($message."\n");
	}
	
	function Error ($message) {
		print($message."\n");
		print("1\n");
	}
	
	// given a valid connection login the user
	function Login () {
		$this->Status("logging in as $this->user...");
		
		// set passive mode in case we are behind a firewall
		ftp_pasv($this->connection, true);
		
		$result = ftp_login($this->connection, $this->user, $this->password);
		
		if ((!$result) and (!$this->connection)) {
			$this->Error("Failed to connect! error: $result");
		} else {
			//$this->Status("Connected with ID: $this->connection");
		}
	}
	
	// creates a new directory
	function CreateDirectory ($directory) {
		if (ftp_mkdir($this->connection, $directory)) {
			$this->Status("Created new directory $directory.");
		} else {
			$this->Error("Error creating the directory $directory.");
		}
	}
	
	// creates a new empty file
	function CreateFile ($where) {
		$handle = fopen("(temp).txt", "w+");
		
		if (ftp_put($this->connection, $where, "(temp).txt", FTP_ASCII)) {
			$this->Status("Created new file $where.");
		} else {
			$this->Error("Error creating the file $where.");
		}
		
		unlink($handle);
	}
	
	// deletes a file
	function DeleteFile ($file) {
		if (ftp_delete($this->connection, $file)) {
			$this->Status("Deleted $file from the server.");
		} else {
			$this->Error("Error deleting the file.");
		}
	}
	
	// deletes a directory
	function DeleteDirectory ($file) {
		
		// remove leading slash for ftp_rawlist
		/*
		if ($file{strlen($file) - 1} == '/') {
			$file = substr($file, 0, strlen($file) - 1);
		}
		*/
		if (ftp_rmdirr($this->connection, $file)) {
			$this->Status("Deleted $file from the server");
		} else {
			$this->Error("Error deleting $file.");
		}
	}
	
	// renames a file or directory
	function Rename ($file, $name) {
		
		if ($file{strlen($file) - 1} == '/') {
			$dirs = explode('/', $file);
			for ($i=0; $i < count($dirs) - 2; $i++) { 
				$newPath .= '/'.$dirs[$i];
			}
			$newPath .= "/$name";
		} else {
			$newPath = substr($file, 0, strrpos($file, '/'));
			$newPath .= "/$name";
		}

		if (ftp_rename($this->connection, $file, $newPath)) {
			$this->Status("Renamed $file to $newPath");
		} else {
			$this->Error("Error renaming $file to $newPath.");
		}
	}
	
	// moves a file or directory
	function Move ($file, $directory) {
		$component = array_pop(explode('/', $file));
		if ($component != "") {
			$directory .= $component;
		} else {
			// the $file is a directory. append the current to the destination
			$dirs = explode('/', $file);
			if ($dirs[count($dirs) - 1] == "") {
				array_pop($dirs);
			}
			
			$directory .= $dirs[count($dirs) - 1];
		}
		
		if (ftp_rename($this->connection, $file, $directory)) {
			$this->Status("Moved $file to $directory");
		} else {
			$this->Error("Error moving $file to $directory.");
		}
	}

	function Chmod ($file, $permissions) {
		if (ftp_chmod($this->connection, intval($permissions), $file)) {
			$this->Status("Set permissions for $file to $permissions.");
		} else {
			$this->Error("Error setting permissions.");
		}
	}
	
	// connect to the server
	function Connect ($server, $user, $password) {
		
		$this->user = $user;
		$this->password = $password;
		
		$this->Status("connecting...");
		
		if ($this->connection == null) {
			$this->connection = ftp_connect($server);
			$this->Login();
		} else {
			$this->Login();
		}
		
	}
}

// map global arguments to local array
foreach($GLOBALS["argv"] as $key => $value) {
	$pair = explode("=", $value);
	if (count($pair) == 2) {
		$input[$pair[0]] = $pair[1];
	}
}
//print_r($input);

// process the action argument
if (isset($input)) {
	$ftp = new TFTP;
	$ftp->Connect($input['server'], $input['user'], $input['password']);
	
	switch ($input['action']) {
		case "chmod": {
			$input['permission'] = "0755";
			$int = (integer)$input['permission'];

			$ftp->Chmod($input['file'], $int);
			break;
		}
		
		case "move": {
			$ftp->Move($input['file'], $input['where']);
			break;
		}

		case "rename": {
			$ftp->Rename($input['file'], $input['name']);
			break;
		}
		
		case "rmdir": {
			$ftp->DeleteDirectory($input['file']);
			break;
		}
		
		case "mkdir": {
			$ftp->CreateDirectory($input['file']);
			break;
		}

		case "mkfile": {
			$ftp->CreateFile($input['file']);
			break;
		}
		case "delete": {
			$ftp->DeleteFile($input['file']);
			break;
		}
	}
	
	$ftp->Status("closing connection...");
	ftp_close($ftp->connection);
}

?>