<?php
	
	// unit for gathering object information
	require_once("class_inspector.php");
	
	//error_reporting(0); //E_ALL
	
	function break_point () {
		trigger_error("break point", E_USER_ERROR);
	}
	
	function WriteError ($line) {
		global $errorLog;
		
		fwrite($errorLog, "$line \n");
		//print("$line \n");
	}
	
	// Maps the parameters from the shell command into the $_GET and $_POST 
	// to simulate forms values
	function MapArgumentsToParameters () {
		
		foreach($GLOBALS["argv"] as $key => $value) {

			$pair = explode("=", $value);
			if (count($pair) == 2) {
				$_POST[$pair[0]] = $pair[1];
				$_GET[$pair[0]] = $pair[1];
			}
			
		}
	}
	
	// converts an array into XML format
	function XMLFormatArray ($parent, $array) {
		foreach($array as $key => $value) {
			
			WriteError("<element>");
			
			// print element
			$type = gettype($value);
			
			// get value string
			if ($type == "object") {
				$valueString = "Object (".get_class($value).")";
			} else {
				$valueString = $value;
			}
			
			WriteError("	<name>$key</name>");
			WriteError("	<type>$type</type>");
			WriteError("	<value>$valueString</value>");
			
			// recurse and refuse to go inside of our parent
			if (is_array($value) and ($key != $parent)) {
				XMLFormatArray($key, $value);
			}
			
			WriteError("</element>");
		}
	}
	
	// converts a var dump into XML format
	function XMLVarDump ($varDump) {
		
		// parse out trigger_error and break_point also if they exist
		// shift out first 2 functions which are RootErrorHandler->StackTrace
		for($x=0; $x<2; $x++) {
			array_shift($varDump);
		}
		
		// pop off the last element which the entry point (include) to the source file
		array_pop($varDump);
		
		foreach($varDump as $entry) {
			
			// iterate entry
			WriteError("	<entry>");
			foreach($entry as $key => $value) {
				if (isset($key)) {
					
					if (is_array($value) && (count($value) > 0)) {
						WriteError("<$key>");
						
						// recurse
						XMLFormatArray($key, $value);
						WriteError("</$key>");
					} 
					
					if (!is_array($value)) {
						WriteError("		<$key>$value</$key>");
					}
				}
			}
			WriteError("	</entry>");
		}
	}
	
	// makes a debug backtrace into XML
	function StackTrace () {
		WriteError("<stacktrace>");
		XMLVarDump(debug_backtrace());
		WriteError("</stacktrace>");
	}
	
	function ErrorHandler ($errno, $errmsg, $filename, $linenum, $vars) {
		global $errorCount;
		
		// map error types to human readable strings
		$errortype = array (
	                E_ERROR              => 'Error',
	                E_WARNING            => 'Warning',
	                E_PARSE              => 'Parsing Error',
	                E_NOTICE             => 'Notice',
	                E_CORE_ERROR         => 'Core Error',
	                E_CORE_WARNING       => 'Core Warning',
	                E_COMPILE_ERROR      => 'Compile Error',
	                E_COMPILE_WARNING    => 'Compile Warning',
	                E_USER_ERROR         => 'User Error',
	                E_USER_WARNING       => 'User Warning',
	                E_USER_NOTICE        => 'User Notice', 
	                //E_STRICT             => 'Runtime Notice',
                	//E_RECOVERABLE_ERROR  => 'Catchable Fatal Error'
	         );
		
		$validErrors = array(E_ERROR, E_PARSE, E_NOTICE, E_WARNING, E_USER_ERROR, E_USER_WARNING, E_USER_NOTICE);
		
		if (in_array($errno, $validErrors)) { 
			
			// open the root element
			if ($errorCount == 0) {
				WriteError("<root>");
			}
			// FOR DEBUGGING ONLY
			//print("--- ERROR: $errmsg ($filename) \n");
			
			// print the class tree if were are on the first error
			if ($errorCount == 0) {
				load_ignored_classes();
				class_print_tree();
				class_inspect_all();
			}
			
			// print the debug trace into the error log
			$errorCount++;
			WriteError("<debugtrace>");
			
			WriteError("<file>$filename</file>");
			WriteError("<line>$linenum</line>");
			WriteError("<message>$errmsg</message>");
			WriteError("<type>$errortype[$errno]</type>");
		
			// write the var dump in XML format
			if (count($vars) > 0) {
				WriteError("<localstack>");
				XMLFormatArray('', $vars);
				WriteError("</localstack>");
			}
		
			// write the stack back trace
			StackTrace();
		
			WriteError("</debugtrace>");
		}
		
		// continue to process errors
		return false;
	}
	
	// Open the error log
	$errorCount = 0;
	$path = "/Library/Logs/php_errors.log";
	$errorLog = fopen($path,"w+");

	$savedHandler = set_error_handler("ErrorHandler");
	
	MapArgumentsToParameters();
	
	// include the requested source file to debug
	if (array_key_exists("source_file", $_POST)) {
		include($_POST["source_file"]);
	}
	
	// close the root element
	if ($errorCount > 0) {
		WriteError("</root>");
	}
	
	// delete the error log if no errors were found
	if ($errorCount == 0) {
		unlink($path);
	}
	
	// close the error log
	fclose($errorLog);
	
?>