<?php

class WebIndexer {
	
	var $handle;					// file handle to the open URL
	var $content;					// content of the loaded page
	var $url;						// current URL of the search
	var $rootURL;					// root URL of the search	
	var $results = array();				// indexed results from the search	
	var $linkExpressions = array();		// expressions in links to index
	var $recurseExpressions = array();		// array of directories to recurse into
	var $indexedPages = array();			// array of indexed pages
	var $parentPage;					// parent (title) of the page being indexed
	
	// returns a results key given a URL
	function GetResultByURL ($url) {
		if ($this->results != null) {
			foreach ($this->results as $key => $value) {
				if ($value == $url) {
					//print("$key\n");
					return $key;
				}
			}
		}
	}
	
	// returns TRUE if the directory matches a sub expression in recurse expressions
	// example: if directory is "home" and expression matches "dirs/go/(home|back)" returns true
	function CanDirectoryRecurse ($directory) {
		foreach ($this->recurseExpressions as $expression) {
			if (ereg($expression, $directory, $captures)) {
				if ($captures[1] == "") {
					return true;
				}
				if ($captures[1] == $directory) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	function AppendResult ($title, $url) {
		
		// prefix the root url if all we have is a directory
		if (!eregi("^http", $url)) {
			// ??? prevent double slashes
			$url = $this->rootURL.$url;
		}
		
		// url is reference append it a parent
		if (eregi("(.*)[#?]+(.*)", $url, $captures)) {
			$key = $this->GetResultByURL($captures[1]);
			if ($key != "") {
				$this->results["$key Pages"][$captures[2]] = $url;
			}
		} else {
			$this->results["$title|$this->parentPage"] = $url;
		}
	}
	
	// prints results in Pascal parser-friendly format
	function PrintResults () {
		foreach ($this->results as $key => $value) {
			print("#CHAPTER:$key|$value\n");
		}
	}
	
	function ParseContent () {
		$regexp = '<a(.*)href="(.*)"(.*)>(.*)<\/a>';
		if (preg_match_all("/$regexp/siU", $this->content, $matches, PREG_SET_ORDER)) {
			
			// evaluate the results
			foreach($matches as $match) {
				
				// validate against our link expressions
				foreach ($this->linkExpressions as $expression) {
					if (eregi($expression, $match[2], $directory)) {
						
						// there was no link expression capture, use the href instead
						if ($directory[1] == "") {
							$directory[1] = $match[2];
						}
						
						// append to results
						$this->AppendResult($match[4], $match[2]);
						
						// if the directory is in the list, recurse. Dont accept urls with # or ?
						if (!eregi("[#?]+", $match[2])) {
							if ($this->CanDirectoryRecurse($directory[1])) {
								$this->LoadPage($match[4], $match[2]);
							}
						}
					}
				}
    			}
		}
	}
	
	// load a page for indexing	
	function LoadPage ($parent, $url) {
		
		// set the root URL
		if ($this->rootURL == "") {
			$this->rootURL = $url;
		}
		
		// prefix the root url if all we have is a directory
		if (!eregi("^http", $url)) {
			$url = $this->rootURL.$url;
		}
		
		// load the page if it hasn't been loaded yet
		if (!in_array($url, $this->indexedPages)) {
			print("#PAGE:$parent|$url\n");
			if ($handle = fopen($url, "r")) {
				$this->handle = $handle;
				$this->url = $url;
				$this->parentPage = $parent;
				
				// append to indexed pages
				array_push($this->indexedPages, $url);
				
				// read the content
				$this->content = "";
		    		while (!feof($this->handle)) {
		        			$part = fread($this->handle, 1024);
		       			$this->content .= $part;
		       			
		       			// ??? report progress to GUI??
		    		}
				
				fclose($handle);
				
				// parse the contents!
				$this->ParseContent();
			}
		}
	}

}

// map global arguments to local array
foreach($GLOBALS["argv"] as $key => $value) {
	$pair = explode("=", $value)
	
	// get array index
	$arrayIndex = explode(":", $pair[0]);
	if (count($arrayIndex) == 2) {
		//print_r($arrayIndex);
		if (!isset($input[$arrayIndex[0]])) {
			$input[$arrayIndex[0]] = array ();
		}
		array_push($input[$arrayIndex[0]], $pair[1]);
		$pair = null;
	}
	
	// append argument
	if (count($pair) == 2) {
		$input[$pair[0]] = $pair[1];
	}
}

if (isset($input)) {
	//print_r($input);
}

//$url = "https://developer.mozilla.org/en/Gecko_DOM_Reference";
//$exp = array ("^https://developer.mozilla.org/en/DOM/(.*)");
//$dirs = array ("^(window|document|element|event|style|range|selection|form|table|tableRow)+");

//$url = "http://xhtml.com/en/xhtml/reference/";
//$exp = array ("^/en/xhtml/reference/(.*)/");
//$dirs = array ();

//$url = "http://xhtml.com/en/css/reference/";
//$exp = array ("^/en/css/reference/(.*)/");
//$dirs = array ();

//Array|Boolean|Date|Error|EvalError|Function|Math|Number|Object|RangeError|ReferenceError|RegExp|String|SyntaxError|TypeError|URIError|Storage
//$url = "https://developer.mozilla.org/En/Core_JavaScript_1.5_Reference";
//$exp = array ("Global_Objects/(.*)", "Global_Properties/(.*)", "Global_Functions/(.*)", "Statments/(.*)", "Operators/(.*)");
//$dir

//$url = "http://th2.php.net/manual/en/";
//$exp = array ("(refs|language|book|function)\.(.*)\.php", "funcref.php", "langref.php");
//$dirs = array ("funcref.php", "langref.php", "book");

$indexer = new WebIndexer ();
$indexer->linkExpressions = $input["Index"];
$indexer->recurseExpressions = $input["Directory"];
$indexer->LoadPage("Home", $input["URL"]);
$indexer->PrintResults();

?>